from django.test import TestCase, SimpleTestCase
from django.core import mail
from model_bakery import baker


class Error403EmailsMiddleware(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.user = baker.make("User", email="some.user@dev.io")

    def setUp(self):
        self.client.force_login(self.user)

    def test_it_sends_email_to_admins(self):
        self.client.get("/secret-area/")
        self.assertEqual(mail.outbox[0].subject, "[Django] Got 403!")
        self.assertIn("juliana.crain@dev.io", mail.outbox[0].to)
        body = mail.outbox[0].body
        self.assertIn("127.0.0.1", body)
        self.assertIn("/", body)
        self.assertIn(self.user.username, body)
        self.assertIn(self.user.email, body)

    def test_it_handles_anon_user(self):
        self.client.logout()
        self.client.get("/secret-area/")


class FailedLoginMiddlewareTest(SimpleTestCase):
    def test_it_sends_email_to_admins(self):
        self.client.post("/login/", {"username": "brute", "password": "force"})
        self.assertEqual(mail.outbox[0].subject, "[Django] Failed login attempt")
        self.assertIn("juliana.crain@dev.io", mail.outbox[0].to)
        body = mail.outbox[0].body
        self.assertIn("127.0.0.1", body)
        self.assertIn("/login/", body)

    def test_it_handles_only_authentication_form_errors(self):
        self.client.post("/tickets/", {"subject": "Website instable"})
        self.assertFalse(mail.outbox)

    def test_it_handles_functional_views_without_forms(self):
        self.client.get("/without-form/")

    def test_it_handles_functional_views_without_context_data(self):
        self.client.get("/without-context-data/")



from http import HTTPStatus
from django.conf import settings
from django.contrib.auth.forms import AuthenticationForm
from django.core.mail import mail_admins


class Error403EmailsMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        if response.status_code == HTTPStatus.FORBIDDEN and not settings.DEBUG:
            path = request.get_full_path()
            ip = request.META.get("REMOTE_ADDR", "<none>")
            if not request.user.is_authenticated:
                mail_admins(
                    "Got 403!",
                    f"Got error 403 on {path} from {ip}. User: Anonymous",
                    fail_silently=True,
                )
                return response
            user = request.user
            mail_admins(
                "Got 403!",
                f"Got error 403 on {path} from {ip}. User: {user.username} - {user.email}",
                fail_silently=True,
            )
        return response


class FailedLoginMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)
        try:
            form = response.context_data.get("form")
            if isinstance(form, AuthenticationForm) and form.error_messages:
                path = request.get_full_path()
                ip = request.META.get("REMOTE_ADDR", "<none>")
                mail_admins(
                    "Failed login attempt",
                    f"Failed login attempt from {ip} on {path}.",
                    fail_silently=True,
                )
        except AttributeError:
            pass
        return response



DEBUG = False
SECRET_KEY = "NOT_SO_SECRET"
DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3"}}
ROOT_URLCONF = "tests.urls"
INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.sessions",
    "monitus",
]
MIDDLEWARE = [
    "monitus.middleware.Error403EmailsMiddleware",
    "monitus.middleware.FailedLoginMiddleware",
    # Session and auth middlewares needed for having request.user in Error403EmailsMiddleware
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
]
ADMINS = [("Juliana C.", "juliana.crain@dev.io")]
EMAIL_BACKEND = "django.core.mail.backends.locmem.EmailBackend"
